﻿--------------------------------------------------------------------------------
--  BUTTON RELATED CODE
--
--[[
	The goal of this code is to have the item buttons be autonomous units,
	allowing the mod to be as flexible as possible.

	inherits="ItemButtonTemplate"
]]--
--------------------------------------------------------------------------------

function FI_Tooltip( self )
	local bid = FI_FrameToID( self:GetName() );
	if FI_SV_CONFIG.debug then print("You hovered button ID: "..bid); end

	local button = FI_DB.select(FI_SVPC_DATA.Buttons, {id = bid}, true);
	
	if button.item then
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT");

		local sName, sLink, iQuality, iLevel, iMinLevel, sType, sSubType, iStackSize = GetItemInfo(button.item);
		if sLink then
			GameTooltip:SetHyperlink(sLink);
			
			if not IsAddOnLoaded("iTip") then
				GameTooltip:AddLine("\n"..sType.." ("..sSubType..")");
				GameTooltip:AddLine("Stack Size:  "..iStackSize, 1,1,1);
				GameTooltip:AddLine("Item Level "..iLevel, 1,1,1);
				GameTooltip:AddLine("Item ID:  "..button.item, 1,1,1);
			end
		end
		
		GameTooltip:AddLine("\n|cFF33CCFF".."Button ID:  "..button.id, 1,1,1);
		
		local counts = "Count:  "..button.count;
		if (iStackSize > 1) then
			local stacks = round((button.count/iStackSize),1);
			counts = counts.."  ("..stacks.." stacks)";
		end
		GameTooltip:AddLine(counts, 1,1,1);
		
		if (button.objective > 0) then
			GameTooltip:AddLine("Objective:  "..button.objective, 1,1,1);
		end
		GameTooltip:AddLine("Include Bank:  "..strupper(tostring(button.bank)));
		if (button.objective > 0) then
			GameTooltip:AddLine("Objective Complete:  "..strupper(tostring(button.success)));
		end
		
		GameTooltip:Show();
	end
end

function FI_Select( bid )
	-- copy to "clipboard"
	FI_SELECTED = bid;

	PlaySound("igAbilityIconPickup");

	-- selection indicator
	_G["FI_Button_"..FI_SELECTED.."_Glow"]:Show();

	if FI_SV_CONFIG.debug then print("Button ID #"..bid.." SELECTED for moving..."); end
end

-- pass true to disable sound
function FI_Deselect( mute )
	-- turn off selection indicator
	_G["FI_Button_"..FI_SELECTED.."_Glow"]:Hide();

	-- clear the "clipboard"
	FI_SELECTED = false;

	if not mute then
		PlaySound("igAbilityIconDrop");
	end
end

-- added in v2.0 beta2
function FI_MoveItem( bid1, bid2 )
	FI_MOVING = true;
	
	local source = FI_DB.select(FI_SVPC_DATA.Buttons, {id = bid1}, true);
	local target = FI_DB.select(FI_SVPC_DATA.Buttons, {id = bid2}, true);
	
	-- move the selected button data
	if FI_SV_CONFIG.debug then print("FI_MoveItem:  Moving ItemID "..source.item); end
	FI_DB.copy(FI_SVPC_DATA.Buttons, {id = source.id}, {id = target.id}); --preserves the primary key ("id" field)
	-- make sure the destination button keeps its *group* id
	FI_DB.update(FI_SVPC_DATA.Buttons, {id = target.id}, {group = target.group}); --do NOT update variable!
	
	-- load new button data in destination (target) slot
	FI_SetButton(target.id);
	
	-- clear original (source) button
	FI_ClearButton(source.id);
	
	-- swapping contents with a populated button?
	if target.item then
		if FI_SV_CONFIG.debug then print("FI_MoveItem:  Swapping places with ItemID "..target.item); end
		
		-- get table index of source button
		local index = FI_DB.find(FI_SVPC_DATA.Buttons, {id = source.id}, true);
		
		-- copy target button data over source button
		FI_SVPC_DATA.Buttons[index] = table.copy(target);
		-- keep the original IDs so things dont get all discombobulated (technical term)
		FI_SVPC_DATA.Buttons[index]["id"] = source.id;
		FI_SVPC_DATA.Buttons[index]["group"] = source.group;
		
		-- load button data
		FI_SetButton(source.id);
	end
	
	FI_Deselect();
	
	FI_MOVING = false;
end

--------------------------------------------------------------------------------
--  CLICK HANDLER
--------------------------------------------------------------------------------
function FI_Click( self, click, down )
	local f_name = self:GetName();
	if FI_SV_CONFIG.debug then print("You CLICKED the "..click.." ("..tostring(down)..") on frame: "..f_name); end

	local button = FI_DB.select(FI_SVPC_DATA.Buttons, {id = FI_FrameToID(f_name)}, true);

	if CursorHasItem() then
		------------------------------------------------------------
		-- NEW ITEM
		------------------------------------------------------------
		local itemType, itemID, itemLink = GetCursorInfo();
		if (itemType == "item") then
			-- save new item to button record
			button = FI_DB.update(FI_SVPC_DATA.Buttons, {id = button.id}, {item = itemID});
			-- apply data to slot
			FI_SetButton(button.id);
			-- clear item from cursor
			ClearCursor();
		end

	elseif (click == "LeftButton") then
		if IsShiftKeyDown() then
			------------------------------------------------------------
			-- INCLUDE BANK
			------------------------------------------------------------
			FI_ToggleBank(button.id);
		
		elseif IsControlKeyDown() then
			------------------------------------------------------------
			-- SET OBJECTIVE
			------------------------------------------------------------
			FI_EditObjective(button.id);
		
		elseif FI_SELECTED then
			------------------------------------------------------------
			-- MOVE
			------------------------------------------------------------
			if (button.id == FI_SELECTED) then
				-- cancel the selection (same slot was clicked)
				FI_Deselect();
			else
				FI_MoveItem(FI_SELECTED, button.id);
			end
			
		elseif button.item then
			------------------------------------------------------------
			-- SELECT ITEM
			------------------------------------------------------------
			FI_Select(button.id);
		end

	elseif (click == "RightButton") then
		if IsShiftKeyDown() then
			------------------------------------------------------------
			-- CLEAR SLOT
			------------------------------------------------------------
			if button.item then
				FI_ClearButton(button.id);
				PlaySound("gsTitleOptionOK");
			end

		else
			------------------------------------------------------------
			-- USE ITEM
			------------------------------------------------------------
			-- Right-click "use" action on actual item is handled by SecureActionButton_OnClick()
			-- See FarmIt2_Button.xml, FI_FRAMES.Button, and FI_SetButton
			if button.item then
				-- make sure we actually have the item in our inventory
				if GetItemSpell(button.item) and (GetItemCount(button.item) > 0) then
					-- inform user that Right-Click action was received
					local itemName, itemLink = GetItemInfo(button.item);
					if itemLink then
						FI_Message("Using "..itemLink);
					end
				end
			end
		end
	end
	-- /mouse buttons
end

--------------------------------------------------------------------------------
--  ITEM RELATED STUFF
--------------------------------------------------------------------------------
function FI_UpdateButton( bid, db_record )
	
	if true or (FI_Uptime() > FI_MIN_UPTIME) or FI_LOADING then
		local button;

		-- streamline button data access when we are doing a global update
		if db_record then
			button = db_record;
		else
			button = FI_DB.select(FI_SVPC_DATA.Buttons, {id = bid}, true);
		end
		
		if button.item then
			if FI_SV_CONFIG.debug and (FI_LOADING == false) and false then
				print("Update running... ButtonID: "..button.id..", ItemID: "..button.item..", Count: "..button.count); --debug
			end
			local f_name = "FI_Button_"..button.id;
			
			--------------------------------------------------------------------------------
			--  CHECK FOR FAILED SERVER QUERIES
			--------------------------------------------------------------------------------
			-- attempt to fix missing icons
			if not _G[f_name.."_Icon"]:GetTexture() then
				if FI_SV_CONFIG.debug then print("Update found a missing texture on BID "..button.id); end
				
				_G[f_name.."_Icon"]:SetTexture( GetItemIcon(button.item) );
			end
			
			-- make sure secure template action has been set
			if not _G[f_name]:GetAttribute("macrotext") then
				if FI_SV_CONFIG.debug then print("Update found missing macrotext on BID "..button.id); end
				
				local itemName = GetItemInfo(button.item);
				if itemName then
					_G[f_name]:SetAttribute("macrotext", "/use "..itemName);
				end
			end
			
			--------------------------------------------------------------------------------
			-- GET CURRENT ITEM COUNT
			--------------------------------------------------------------------------------
			local newcount = GetItemCount(button.item, button.bank);
			
			-- try to be smart about only running interface and data changes when we need to
			if FI_LOADING or FI_MOVING or (newcount ~= button.count) then
				--------------------------------------------------------------------------------
				-- SAVE NEW ITEM COUNT
				--------------------------------------------------------------------------------
				local query = {
					["count"] = newcount,
					["lastcount"] = button.count,
				}
				button = FI_DB.update(FI_SVPC_DATA.Buttons, {id = button.id}, query);
				
				-- update graphical counter
				_G[f_name.."_Count"]:SetText(button.count);
				
				--------------------------------------------------------------------------------
				--  PROGRESS TRACKING
				--------------------------------------------------------------------------------
				FI_Progress(button);
			end
			
		else
			-- no item, zero the counter
			_G["FI_Button_"..button.id.."_Count"]:SetText(0);
		end
	end
end

function FI_SetButton( bid, newItem )
	local button;
	
	-- update itemID before setting button elements
	if newItem then
		FI_ClearButton(bid);
		button = FI_DB.update(FI_SVPC_DATA.Buttons, {id = bid}, {item = newItem});
	else
		button = FI_DB.select(FI_SVPC_DATA.Buttons, {id = bid}, true);
	end
	
	if button.item then
		local f_name = "FI_Button_"..button.id;

		-- set icon
		_G[f_name.."_Icon"]:SetTexture( GetItemIcon(button.item) ); --if this fails, FI_UpdateButton should fix it

		-- bank inclusion indicator
		if button.bank then
			_G[f_name.."_Bank"]:Show();
		end
		
		-- set secure template action
		local itemName, itemLink = GetItemInfo(button.item); --if this fails, FI_UpdateButton should fix it
		if itemName then
			_G[f_name]:SetAttribute("macrotext", "/use "..itemName);
		end
		
		-- handle objective
		if (button.objective == 0) or CursorHasItem() then
			-- no objective -OR- we are in the middle of placing a new item
			FI_ClearObjective(button.id);
		else
			-- there is an objective, apply it to the interface
			_G[f_name.."_Objective"]:SetText(button.objective);
			
			local color;
			if (button.count < button.objective) then
				color = FI_SV_CONFIG.Colors.objective;
			else
				color = FI_SV_CONFIG.Colors.success;
			end
			_G[f_name.."_Objective"]:SetVertexColor(color[1], color[2], color[3]);
			
			_G[f_name.."_Objective"]:Show();
		end
		
		FI_UpdateButton(nil, button);
		
		if FI_SV_CONFIG.debug then
			if itemLink then
				print( "FI_SetButton:  Button ID "..button.id.." set to "..itemLink ); --debug
			else
				print( "FI_SetButton:  Button ID "..button.id.." GetItemInfo FAILED!"); --debug
			end
		end
	end
end

function FI_ClearButton( bid )
	local button = FI_DB.select(FI_SVPC_DATA.Buttons, {id = bid}, true);
	
	-- reset button data
	local i = FI_DB.find(FI_SVPC_DATA.Buttons, {id = button.id}, true);
	FI_SVPC_DATA.Buttons[i] = table.copy(FI_DEFAULTS.DB.Button);
	-- preserve the ID's
	FI_SVPC_DATA.Buttons[i]["id"] = button.id;
	FI_SVPC_DATA.Buttons[i]["group"] = button.group;

	FI_ClearObjective(button.id);
	
	-- reset graphical elements
	local f_name = "FI_Button_"..button.id;
	_G[f_name.."_Icon"]:SetTexture("");
	_G[f_name.."_Count"]:SetText(0);
	_G[f_name.."_Bank"]:Hide();
	
	-- clear secure template action
	_G[f_name]:SetAttribute("macrotext", nil);
end

--------------------------------------------------------------------------------
--  BANK INVENTORY
--------------------------------------------------------------------------------
function FI_ToggleBank( bid )
	local button = FI_DB.select(FI_SVPC_DATA.Buttons, {id = bid}, true);
	local f_name = "FI_Button_"..button.id;
	
	if button.item then
		-- change setting
		button = FI_DB.update(FI_SVPC_DATA.Buttons, {id = button.id}, {bank = FI_Toggle(button.bank)});
		
		-- bank indicator
		if button.bank then
			_G[f_name.."_Bank"]:Show();
		else
			_G[f_name.."_Bank"]:Hide();
		end
		
		-- refresh item count
		FI_UpdateButton(button.id);
		
		PlaySound("TalentScreenClose");
		FI_Message("Button ID "..button.id..":  Include Bank = "..strupper(tostring(button.bank)));
	end
end
